var Class = require("core").Class;

var RenderingController = require("./RenderingController.js").RenderingController;

/**
 * @class RequestProcessor
 * @requires Class
 * @requires RenderingController
 * @param {http.ServerRequest} request
 * @param {http.ServerResponse} response
 * @throws {RequestProcessor.Exception.INVALID_REQUEST_RESPONSE} if constructor parameters are invalid or null.
 */
var RequestProcessor = function(){
    /**
     * Request object
     * @public 
     * @type {http.ServerRequest}
     */
    this.request = null;
    /**
     * Response object
     * @public
     * @type {http.ServerResponse}
     */
    this.response = null;
    /**
     * holds the request id
     * @private 
     * @type {mixed}
     */
    this._id;
    
    /** @ignore */
    this.initialize = function(request, response){


        var http = require('http');

        if(!request || !response){
            throw RequestProcessor.Exception.INVALID_REQUEST_RESPONSE;
        }
        else if(!(request instanceof http.IncomingMessage)){
            throw RequestProcessor.Exception.INVALID_REQUEST_RESPONSE;
        }
        else if(!(response instanceof http.ServerResponse)){
            throw RequestProcessor.Exception.INVALID_REQUEST_RESPONSE;
        }

        console.time('   -- REQUEST TIME');
        this.request = request;
        this.response = response;
    };
    /**
     * Starts processing the request. Creates and dispatches rendering flow.
     * @public
     */
    this.process = function(){
        //FIXME: dodac sprawdzanie czy format zapytania jest zgodny z jsonrpc - dla zapewnienia kompatybilnosci ze standardem
        //sprawdzam czy podanego json da sie sparsowac
        try{
            var request = JSON.parse(this.request.data);
        } catch(error){
            this.sendResponse({
                error: {
                    code: -32700,
                    message: "Invalid JSON. An error occurred on the server while parsing the JSON text.",
                    data: error
                }
            });
            return;
        }
        //weryfikuje poprawnosc zapytania
        if(!request.hasOwnProperty("method") ||
           !request.hasOwnProperty("jsonrpc") || request.jsonrpc != "2.0" ||
           !request.hasOwnProperty("params") ||
           !request.hasOwnProperty("id") || request.id == ""){
            this.sendResponse({
                error: {
                    code: -32600,
                    message: "The received JSON is not a valid JSON-RPC Request."
                }
            });
            return;
        }else{
            this._id = request.id;
        }
        
        //weryfikuje metode
        if(request.method != "render"){
            this.sendResponse({
                error: {
                    code: -32601,
                    message: "The requested remote-procedure does not exist / is not available."
                }
            });
            return;
        }
        
        //weryfikuje poprawnosc konfiguracji
        try{
            var controller = new RenderingController(request.params);            
        }catch(e){
            this.sendResponse({
                error: {
                    code: -32602,
                    message: "Invalid method parameters.",
                    data: e
                }
            });
            return;            
        }

        //wywalam ogolny blad jesli nie zostal przez nic glebiej przechwycony
        try{
            controller.addEventListener(RenderingController.RENDERED, this._onRenderingSuccess, this);
            controller.addEventListener(RenderingController.ERROR, this._onRenderingError, this);
            controller.render();
        }catch(e){
            this.sendResponse({
                error: {
                    code: -32603,
                    message: "Internal JSON-RPC error.",
                    data: e
                }
            });
            return;  
        }
    };
    /**
     * Called on rendering success
     * 
     * @param {Event} e
     * @private
     */
    this._onRenderingSuccess = function(e){
        this.sendResponse({
            result: e.data
        });
    };
    /**
     * Called on rendering error
     * 
     * @param {ErrorEvent} e
     * @private
     */
    this._onRenderingError = function(e){
        this.sendResponse({
            error: {
                code: -123,
                message: e.toHtmlString()
            }
        });
    };
    /**
     * Sends response
     *
     * @param {Object} data Response data.
     */
    this.sendResponse = function(data){
        console.timeEnd('   -- REQUEST TIME');
        this.response.writeHead(200, {
            'Content-Type': 'application/json; charset=utf-8',
            'Access-Control-Allow-Origin': '*'
        });

        this.response.end(new Buffer(JSON.stringify(this._merge({
            id: this._id,
            jsonrpc: "2.0"
        }, data))));
        console.log('-- request ended');
    };
    
    /**
     * helper method, merges two or more object into one
     * 
     * @param {object}
     * @private
     * 
     * @returns {object} merged object
     */
    this._merge = function(){
        var tmp = {};
        
        for(var i = 0, l = arguments.length; i < l; i++){
            for(var key in arguments[i]){
                if(arguments[i].hasOwnProperty(key)){
                    tmp[key] = arguments[i][key];
                }
            }
        }
        
        return tmp;
    };
};

RequestProcessor = new Class( new RequestProcessor() );

/**
 * Namespace for exeptions messages.
 * @static
 * @constant
 * @namespace
 */
RequestProcessor.Exception = {};
/**
 * @static
 * @constant
 */
RequestProcessor.Exception.INVALID_REQUEST_RESPONSE = "Invalid Request and/or Response objects";

exports.RequestProcessor = RequestProcessor;