var Class = require("core").Class;
var DriverAbstract = require('./DriverAbstract.js').DriverAbstract;

/**
 * @class Driver Manager Responsible for managing drivers (creating, adding, fetching). 
 * @requires Class
 * @requires DriverAbstract
 */
var DriverManager = function(){

    /**
     * array of drivers to handle
     *
     * @type Array[DriverAbstract]
     * @private
     */
    this._drivers = [];

    /**
     * Adds a driver to collection
     * @public
     * @param {DriverAbstract} driver Driver instance.
     * @throws {DriverManager.Exception.WRONG_INSTANCE} if given driver is not an instance of DriverAbstract class.
     * @returns {boolean} Returns {false} if given driver is arledy in collection otherwise returns {true}
     */
    this.addDriver = function(driver){
        if(driver instanceof DriverAbstract) {
            for(var i = 0, l = this._drivers.length; i < l; i++){
                if(this._drivers[i] == driver){
                    return false;
                }
            }

            this._drivers.push(driver);
            return true;
        }
        else {
            throw DriverManager.Exception.WRONG_INSTANCE;
        }
    };

    /**
     * Loads model from each added driver. See: {@link DriverAbstract#load}
     * @public
     */
    this.fetch = function(){
        for(var i=0,max=this._drivers.length;i<max;i++){
            this._drivers[i].load();
        }
    };
};

DriverManager = new Class(new DriverManager());

/**
 * Factory method - createing new drivers of given <i>driverClassName</i>.
 *
 * @static
 * @param {String} driverClassName Name of a driver class.
 * @param mixed data Data passed to the driver's constructor
 * @return {DriverAbstract}
 * @throws {DriverManager.Exception.DRIVER_DOES_NOT_EXIST} if given driver class name cannot be resolved.
 */
DriverManager.createDriver = function(driverClassName, data){
    //TODO przerbic domyslną sciezke
    try{
        var driverLib = require('./' + driverClassName + '.js');
        var DriverClass = driverLib[driverClassName];
        var driver = new DriverClass(data);
        return driver;
    }catch(e){
        throw DriverManager.Exception.DRIVER_DOES_NOT_EXIST;
    }
};

/**
 * @constant
 * @static
 */
DriverManager.LOADED = "DriverManager_LOADED";
/**
 * @constant
 * @static
 */
DriverManager.ERROR = "DriverManager_ERROR";

/**
 * Namespace for exeptions messages.
 * @constant
 * @static
 * @namespace
 */
DriverManager.Exception = {};
/**
 * @constant
 * @static
 */
DriverManager.Exception.WRONG_INSTANCE = 'Driver must be instance of DriverAbstract class.';
/**
 * @constant
 * @static
 */
DriverManager.Exception.DRIVER_DOES_NOT_EXIST = 'Given driver does not exist or there is an require error in driver class.';

exports.DriverManager = DriverManager;