var Class           = require('core').Class;
var JsonRpcResponse = require('core').JsonRpc.JsonRpcResponse;
var JsonRpcError    = require('core').JsonRpc.JsonRpcError;
var JsonRpcAbstract = require('core').JsonRpc.JsonRpcAbstract;
var EventDispatcher = require('core').Event.EventDispatcher;
var Event           = require('core').Event.Event;

/**
 * Representing template.
 * @class Template Base template class. 
 * @extends EventDispatcher
 * @requires Class
 * @requires Event 
 * @requires EventDispatcher
 * @requires JsonRpcAbstract
 * @requires JsonRpcError  
 * @requires JsonRpcResult
 * @requires TemplateManager
 * @param {String} name
 * @param {Object} configuration
 * @throws {Template.Exception.CONFIG_INVALID} if given <i>name</i> or <i>configuration</i> are invalid.
 */
var Template = function(){
    /** @ignore */
    this.Extends = EventDispatcher;
   
    /**
     * Indicates if template is a child of parent.
     * @public
     * @type {Boolean}
     */
    this.hasParent = false;
    /**
     * Indicates if template was rendered.
     * @public
     * @type {Boolean}
     */
    this.isRendered = false;
    
    /**
     * configuration object
     * @private
     * @type {Object}
     */
    this._configuration = null;
    /**
     * generated content
     * @private
     * @type {Object}
     */
    this._content = null;
    /**
     * rendered children counter
     * @private
     * @type {Number}
     */ 
    this._counter = 0; 
    /**
     * template model
     * @private
     * @type {Object}
     */
    this._data = null;
    /**
     * template name within configuration
     * @private
     * @type {Boolean}
     */        
    this._name = "";


    /** @ignore */
    this.initialize = function(name, configuration){

        if(!name || !configuration || !configuration.hasOwnProperty("template")){
            throw Template.Exception.CONFIG_INVALID;
        }
        //public
        this.hasParent = false;
        this.isRendered = false;
        //private
        this._configuration = configuration;
        this._content = {
            result: null
        }; 
        this._counter = 0; 
        this._data = null;
        this._name = name;
    };
    
    /**
     * Abstract method. Needed to be overriden - otherwise Template.Exception.NOT_IMPLEMENTED will be throwed.
     * @public
     * @param {Object} data
     * @throw {Template.Exception.NOT_IMPLEMENTED}
     */
    this.render = function(data){
        throw Template.Exception.NOT_IMPLEMENTED;
    };
    
    /**
     * Dispatchig render action and handles its errors and exceptions.
     * @public
     * @param {Object} data
     * @throws {Template.Exception.NOT_IMPLEMENTED} if method render wasn't overriden 
     */
    this.dispatchRender = function(data){
        try{
            console.log('   -- Template '+this._name+': rendering...');
            this.render(data);
        }
        catch(e){
            console.log('   -- Template '+this._name+': error while rendering...', e);
            if(e == Template.Exception.NOT_IMPLEMENTED){
                 //fallback in case of not implementing
                throw Template.Exception.NOT_IMPLEMENTED;
            }
            this._handleError(e);
        }
    };
    
    /**
     * Handling model load. Called on DriverAbstract.LOADED
     * @public
     * @param {Event} e
     */
    this.onData = function(e){
        if(this._templateManager.isAborted()){
            return;
        }
        
        console.log('   -- Template '+this._name+': received data');
        this._data = e.data;

        if(this._data && this._data.getResult().hasOwnProperty('#body')){
            var children = this._data.getResult()['#body'];
            this._counter = children.length;
        
            console.log('   -- Template '+this._name+': have '+children.length +' children');
        
            for( var i = 0, max = children.length; i < max; i++ ){
                console.log('   -- Template '+this._name+': waiting for children: '+children[i]);
                var child = this.getTemplateManager().getTemplateByName(children[i]);
                if(child) {
                    try {
                        this.addChild(child);    
                    }
                    catch(err) {
                        this._handleError(err);
                    }
                }
                else {
                    this._counter--;
                }
            }            
        }else{
            console.log('   -- Template '+this._name+': does\'nt have children');
            this.dispatchRender(this._data);
        }
    };
        
    /**
     * Handling model error. Called on DriverAbstract.ERROR
     * @public
     * @param {ErrorEvent} e
     */
    this.onDataError = function(e){
        console.log('   -- Template '+this._name+': data error');
        //TODO handle error
    };
    
    /**
     * Adding child template
     * @public
     * @param {Template} child
     * @throws {Template.Exception.RECURSIVE_TEMPLATES} if given child is a self instance.
     */
    this.addChild = function(child){
        if(child === this){
            throw Template.Exception.RECURSIVE_TEMPLATES;
        }
        child.addEventListener(Template.RENDERED, this._onTemplateRendered, this);
        child.hasParent = true;
        if(child.isRendered){
            this._onTemplateRendered(new Event(Template.RENDERED, child));
        }
    };
    
    /**
     * Setting rendering result
     * @public
     * @param {Object} data 
     */
    this.setRenderedData = function(data){
        console.log('   -- Template '+this._name+': rendered.');
        
        if(!data){
            throw Template.Exception.WRONG_RENDER_DATA;
        }
        else if(typeof data == 'string'){
            this._content = new JsonRpcResponse();
            this._content.setResult(data);
        }
        else if(data instanceof JsonRpcAbstract){
            this._content = data;
        }
        else {
            throw Template.Exception.WRONG_RENDER_DATA;
        }

        console.log('   -- Template '+this._name+': dispaching...');
        this.isRendered = true;
        this.dispatchEvent(new Event(Template.RENDERED, this));
    };
    
    /**
     * Returns content generated by specific render engine
     * @public
     * @return {String}
     */
    this.getContent = function(){
        return this._content;
    };
    
    /**
     * Returns error handler
     * @public
     * @return {CONST}
     */
    this.getErrorAction = function(){
        return this._configuration.onerror ? this._configuration.onerror : Template.ACTION_ERROR_DEFAULT;
    };

    /**
     * Returns template manager who holds every template in the subtree
     * @public
     * @return {TemplateManager}
     */
    this.getTemplateManager = function(){
        if(!this._templateManager){
            throw Template.Exception.TEMPLATEMANAGER_NULL;
        }
        return this._templateManager;
    };

    /**
     * Sets template manager object which holds every template in the subtree
     * @public
     * @param {TemplateManager} templateManager
     */
    this.setTemplateManager = function(templateManager){
        
        var TemplateManager = require('../TemplateManager.js').TemplateManager;

        if(!templateManager){
            throw Template.Exception.TEMPLATEMANAGER_NULL;
        }
        else if(!(templateManager instanceof TemplateManager)){
           throw Template.Exception.TEMPLATEMANAGER_NOT_CLASS_INSTANCE;
           return;
        }
        this._templateManager = templateManager;
    };

    /**
     * Returns template name
     * @public
     * @return {String}
     */
    this.getName = function(){
        return this._name;
    };

    /**
     * Returns template location path
     * @public
     * @return {String} template path
     */
    this.getPath = function(){
        return this._configuration.template;
    };
    
    /**
     * Callback function called on rendered child template
     * @private
     * @param {Event} e
     */
    this._onTemplateRendered = function(e){
        this._counter--;

        console.log('   -- Template '+this._name+': received children: '+e.data.getName());

        if(this._counter === 0){

            console.log('   -- Template '+this._name+': have all children');
        
            if(this._data.getResult().hasOwnProperty('#body')){
                var children = this._data.getResult()['#body'];
                this._data._result.body = [];

                for(var i=0,max=children.length;i<max;i++){
                    this._data._result.body.push(this.getTemplateManager().getTemplateByName(children[i]).getContent());
                }
                delete this._data._result['#body'];
            }
            this.dispatchRender(this._data);
        } else {
            console.log('   -- Template '+this._name+': children left: '+this._counter);
        }
    };
    
    /**
     * Handling error
     * @private
     */
     this._handleError = function(e){
            switch(this.getErrorAction()){
                case Template.ACTION_ERROR_PROPAGATE:
                    var response = new JsonRpcResponse();
                    response.setError(JsonRpcError.factory(Template.Error.PROPAGATED).setData(e));
                    this.setRenderedData(response);
                    break;
                case Template.ACTION_ERROR_DISCARD:
                    this.setRenderedData(new JsonRpcResponse());
                    break;
                case Template.ACTION_ERROR_ABORT:
                    this._templateManager.abort(JsonRpcError.factory(Template.Error.ABORTED).setData(e));
                    break;    
            }
     };
};

Template = new Class( new Template() );

/**
 * @static
 * @constant
 */
Template.RENDERED = "Template_RENDERED";
/**
 * @static
 * @constant
 */
Template.ERROR = "Template_ERROR";
/**
 * @static
 * @constant
 */
Template.ACTION_ERROR_PROPAGATE = "propagate";
/**
 * @static
 * @constant
 */
Template.ACTION_ERROR_DISCARD = "discard";
/**
 * @static
 * @constant
 */
Template.ACTION_ERROR_ABORT = "abort";
/**
 * @static
 * @constant
 */
Template.ACTION_ERROR_DEFAULT = Template.ACTION_ERROR_DISCARD;
/**
 * Namespace for exeptions messages.
 * @static
 * @constant
 * @namespace
 */
Template.Exception = {};
/**
 * @static
 * @constant
 */
Template.Exception.CONFIG_INVALID = "Error - Template name and configuration are invalid.";
/**
 * @static
 * @constant
 */
Template.Exception.NOT_IMPLEMENTED = "Method:render() - Not implemented!";
/**
 * @static
 * @constant
 */
Template.Exception.TEMPLATEMANAGER_NULL = "TemplateManager is null";
/**
 * @static
 * @constant
 */
Template.Exception.TEMPLATEMANAGER_NOT_CLASS_INSTANCE = "templateManager must be instance of TemplateManager class";
/**
 * @static
 * @constant
 */
Template.Exception.FILE_DOES_NOT_EXIST = function(fileName){
    return "Template: " + fileName + " doesn't exist";
};
/**
 * @static
 * @constant
 */
Template.Exception.WRONG_RENDER_DATA = "Given data has to be String";
/**
 * @static
 * @constant
 */
Template.Exception.RECURSIVE_TEMPLATES = "Template cannot be his own child!!!";
/**
 * Namespace for error objects;
 * @static
 * @constant
 * @namespace
 */
Template.Error = {};
/**
 * @static
 * @constant
 */
Template.Error.ABORTED = new JsonRpcError(-2201, "Aborting because of template error");
/**
 * @static
 * @constant
 */
Template.Error.PROPAGATED = new JsonRpcError(-2202, "Error occured, propagating error.");

exports.Template = Template;