describe('RenderingController', function(){

    var RenderingController = require('../lib/RenderingController').RenderingController;
    var Template = require('../lib/template/Template.js').Template;
    var TemplateManager = require('../lib/TemplateManager.js').TemplateManager;
    var DriverManager = require('../lib/DriverManager.js').DriverManager;

    var jqtpl = require('jqtpl');
    
    var generatedContent = {};

    beforeEach(function(){
        jqtpl.template("container001.jqtpl", "CONTAINER1{{each data.body}}{{if data.body[$index].hasOwnProperty('error') }} <b>{{html $value.error.message}}</b>{{else}}{{html $value.result}}{{/if}}{{/each}}");
        jqtpl.template("title001.jqtpl", "<h2>${data}</h2>");
        
        generatedContent.CorrectConfiguration = "CONTAINER1<h2>Tytul bloczka1</h2><h2>Tytul bloczka2</h2>";
    });

    it("costructor", function(){
        expect(function(){ new RenderingController(); }).toThrow(RenderingController.Exception.WRONG_CONFIGURATION);
        expect(function(){ new RenderingController(null); }).toThrow(RenderingController.Exception.WRONG_CONFIGURATION);
        expect(function(){ new RenderingController(undefined); }).toThrow(RenderingController.Exception.WRONG_CONFIGURATION);
        expect(function(){ new RenderingController(123); }).toThrow(RenderingController.Exception.WRONG_CONFIGURATION);
        expect(function(){ new RenderingController("eloszka"); }).toThrow(RenderingController.Exception.WRONG_CONFIGURATION);
        expect(function(){ new RenderingController({}); }).toThrow(RenderingController.Exception.WRONG_CONFIGURATION);
        expect(function(){ new RenderingController(new (function(){})); }).toThrow(RenderingController.Exception.WRONG_CONFIGURATION);

        expect(function(){ new RenderingController([]); }).not.toThrow();        
    });

    xit("WrongTemplateType", function(){
        var rc = new RenderingController(_getConfigFromFile('RenderingController/WrongTemplateType.json'));
        expect(function(){
            rc.render();
        }).toThrow(TemplateManager.Exception.UNRESOLVED_TYPE);
    });
    
    it("NoLocalDriverSpecified", function(){
        var rc = new RenderingController(_getConfigFromFile('RenderingController/NoLocalDriverSpecified.json'));
        expect(function(){
            rc.render();
        }).not.toThrow(/*'Cannot create data driver'*/);
    });
    
    it("NoRemoteDriverSpecified", function(){
        var rc = new RenderingController(_getConfigFromFile('RenderingController/NoRemoteDriverSpecified.json'));
        expect(function(){
            rc.render();
        }).toThrow(RenderingController.Exception.NO_REMOTE_DRIVER);
    });
    
    it("WrongRemoteDriver", function(){
        var rc = new RenderingController(_getConfigFromFile('RenderingController/WrongRemoteDriver.json'));
        expect(function(){
            rc.render();
        }).toThrow(DriverManager.Exception.DRIVER_DOES_NOT_EXIST);
    });
    
    it("CorrectConfiguration", function(){
        var response = null;

        runs(function(){
            var rc = new RenderingController(_getConfigFromFile('RenderingController/CorrectConfiguration.json'));
            rc.addEventListener(RenderingController.RENDERED, function(e){ response = e;});
            rc.addEventListener(RenderingController.ERROR, function(e){ response = e;});
            rc.render();
        });
        
        waitsFor(function(){
            return response !== null;
        }, 1000);
        
        runs(function(){
            expect(response.type).toEqual(RenderingController.RENDERED);
            expect(response.data[0].c001.result).toEqual(generatedContent.CorrectConfiguration);
        });
    });
    
    it("RevertedConfiguration", function(){
        var response = null;

        runs(function(){
            var configuration = _getConfigFromFile('RenderingController/RevertedConfiguration.json');
            var rc = new RenderingController(configuration);
            rc.addEventListener(RenderingController.RENDERED, function(e){ response = e;});
            rc.addEventListener(RenderingController.ERROR, function(e){ response = e;});
            rc.render();
        });
        
        waitsFor(function(){
            return response !== null;
        }, 1000);
        
        runs(function(){
            expect(response.type).toEqual(RenderingController.RENDERED);
        });
    });
    
    it("WrongRecursiveContainer", function(){
        var response = null;

        runs(function(){
            var rc = new RenderingController(_getConfigFromFile('RenderingController/WrongRecursiveContainer.json'));
            rc.addEventListener(RenderingController.RENDERED, function(e){ response = e;});
            rc.addEventListener(RenderingController.ERROR, function(e){ response = e;});
            rc.render();
        });
        
        waitsFor(function(){
            return response !== null;
        }, "seems like we've timed out", 1000);
        
        runs(function(){
            //console.log(response.data[0].c001.error.code);
            expect(response.type).toEqual(RenderingController.RENDERED);
            expect(response.data[0].c001.error.code).toEqual(-2101);
        });
    });
    
    it("TemplateInTemplate", function(){
        var response = null;

        runs(function(){
            var rc = new RenderingController(_getConfigFromFile('RenderingController/TemplateInTemplate.json'));
            rc.addEventListener(RenderingController.RENDERED, function(e){ response = e;});
            rc.addEventListener(RenderingController.ERROR, function(e){ response = e;});
            rc.render();
        });
        
        waitsFor(function(){
            return response !== null;
        }, 1000);
        
        runs(function(){
            expect(response.type).toEqual(RenderingController.RENDERED);
        });
    });
    
    it("FiveRecursiveContainersAndATemplate", function(){
        var response = null;

        runs(function(){
            var rc = new RenderingController(_getConfigFromFile('RenderingController/FiveRecursiveContainersAndATemplate.json'));
            rc.addEventListener(RenderingController.RENDERED, function(e){ response = e;});
            rc.addEventListener(RenderingController.ERROR, function(e){ response = e;});
            rc.render();
        });
        
        waitsFor(function(){
            return response !== null;
        }, 1000);
        
        runs(function(){
            expect(response.type).toEqual(RenderingController.RENDERED);
        });
    });
    
    it("NoTemplateForContainerPropagate", function(){
        var config, response = null;

        runs(function(){
            config = _getConfigFromFile('RenderingController/NoTemplateForContainerPropagate.json');

            var rc = new RenderingController(config);
            rc.addEventListener(RenderingController.RENDERED, function(e){ response = e;});
            rc.addEventListener(RenderingController.ERROR, function(e){ response = e;});
            rc.render();
        });
        
        waitsFor(function(){
            return response !== null;
        }, 1000);
        
        runs(function(){
            expect(response.type).toEqual(RenderingController.RENDERED);
            expect(response.data[0].c001.getData()).toEqual(Template.Exception.FILE_DOES_NOT_EXIST(config[0].c001.template));
        });
    });
    //TODO
    xit("NoTemplateForContainerDiscard", function(){
        var config, response = null;

        runs(function(){
            config = _getConfigFromFile('RenderingController/NoTemplateForContainerDiscard.json');

            var rc = new RenderingController(config);
            rc.addEventListener(RenderingController.RENDERED, function(e){ response = e;});
            rc.addEventListener(RenderingController.ERROR, function(e){ response = e;});
            rc.render();
        });
        
        waitsFor(function(){
            return response !== null;
        }, 1000);
        
        runs(function(){
            expect(response.type).toEqual(RenderingController.RENDERED);
            expect(response.data[0].c001).toEqual("");
        });
    });
    
    it("NoTemplateForContentPropagate", function(){
        var config, response = null;

        runs(function(){
            config = _getConfigFromFile('RenderingController/NoTemplateForContentPropagate.json');

            var rc = new RenderingController(config);
            rc.addEventListener(RenderingController.RENDERED, function(e){ response = e;});
            rc.addEventListener(RenderingController.ERROR, function(e){ response = e;});
            rc.render();
        });
        
        waitsFor(function(){
            return response !== null;
        }, 1000);
        
        runs(function(){
            expect(response.type).toEqual(RenderingController.RENDERED);
            expect(response.data[0].t001.getData()).toEqual(Template.Exception.FILE_DOES_NOT_EXIST(config[0].t001.template));
        });
    });
    //TODO
    xit("NoTemplateForContentDiscard", function(){
        var config, response = null;

        runs(function(){
            config = _getConfigFromFile('RenderingController/NoTemplateForContentDiscard.json');

            var rc = new RenderingController(config);
            rc.addEventListener(RenderingController.RENDERED, function(e){ response = e;});
            rc.addEventListener(RenderingController.ERROR, function(e){ response = e;});
            rc.render();
        });
        
        waitsFor(function(){
            return response !== null;
        }, 1000);
        
        runs(function(){
            expect(response.type).toEqual(RenderingController.RENDERED);
            expect(response.data[0].t001).toEqual("");
        });
    });
    
    xit("NoTemplateForStylesheetPropagate", function(){
        var config, response = null;
        
        runs(function(){
            config = _getConfigFromFile('RenderingController/NoTemplateForStylesheetPropagate.json');

            var rc = new RenderingController(config);
            rc.addEventListener(RenderingController.RENDERED, function(e){ response = e;});
            rc.addEventListener(RenderingController.ERROR, function(e){ response = e;});
            rc.render();
        });
        
        waitsFor(function(){
            return response !== null;
        }, 1000);
        
        runs(function(){
            expect(response.type).toEqual(RenderingController.RENDERED);
            expect(response.data[0].s001).toEqual(Template.Exception.FILE_DOES_NOT_EXIST(config[0].s001.template));
        });
    });
    
    xit("NoTemplateForStylesheetDiscard", function(){
        var config, response = null;

        runs(function(){
            config = _getConfigFromFile('RenderingController/NoTemplateForStylesheetDiscard.json');

            var rc = new RenderingController(config);
            rc.addEventListener(RenderingController.RENDERED, function(e){ response = e;});
            rc.addEventListener(RenderingController.ERROR, function(e){ response = e;});
            rc.render();
        });
        
        waitsFor(function(){
            return response !== null;
        }, 1000);
        
        runs(function(){
            expect(response.type).toEqual(RenderingController.RENDERED);
            expect(response.data[0].s001).toEqual("");
        });
    });
});