describe('Template', function(){

    var Template = require('../../lib/template/Template.js').Template;
    var JsonRpcAbstract = require('../../lib/core/jsonrpc/JsonRpcAbstract.js').JsonRpcAbstract;
    var Class = require('../../lib/mootools/mootools-node.js').Class;

    var JsonRpcMock = null;
    
    it('creation', function(){
        //null name and configuration
        expect(function(){
            var tpl = new Template();
        }).toThrow(Template.Exception.CONFIG_INVALID);
        
        //undefined configuration
        expect(function(){
            var tpl = new Template("testName");
        }).toThrow(Template.Exception.CONFIG_INVALID);
        
        //wrong name
        expect(function(){
            var tpl = new Template(null, {});
        }).toThrow(Template.Exception.CONFIG_INVALID); 
        
        //empty configuration
        expect(function(){
            var tpl = new Template("testName", {});
        }).toThrow(Template.Exception.CONFIG_INVALID);
        
        //proper use
        expect(function(){
            var tpl = new Template("testName", {template:"example/path"});
        }).not.toThrow(Template.Exception.CONFIG_INVALID); 
    });
    
    it('creating mock jsonrpcabstract class', function(){
        expect(function(){
            JsonRpcMock = function(){
                this.Extends = JsonRpcAbstract;

                this._data = null;
                
                this.initialize = function(data){
                    this._data = data;
                };
                
                this.toJson = function(){
                    var result = {};
                    result.data = this._data
                    return result;
                };
            };
            JsonRpcMock = new Class(new JsonRpcMock());
        }).not.toThrow();
    });
    
    it('creation - configuration reading / defaults', function(){
        var sampleConfiguration = {
            template:"example/path",
            onerror: Template.ACTION_ERROR_DISCARD    
        };
        var tpl = new Template("testName", sampleConfiguration);
        
        expect(tpl.getPath()).toEqual(sampleConfiguration.template);
        expect(tpl.getErrorAction()).toEqual(sampleConfiguration.onerror);
        expect(tpl.hasParent).toBeFalsy();
        expect(tpl.isRendered).toBeFalsy();
    });
    it('method render', function(){
        var sampleConfiguration = {
            template:"example/path"
        };
        var tpl = new Template("testName", sampleConfiguration);
        
        expect(function(){
            tpl.render();
        }).toThrow(Template.Exception.NOT_IMPLEMENTED);
    });
    it('dispatchRender', function(){
        console.log("dispatchRender");
        var sampleConfiguration = {
            template:"example/path"
        };
        var tpl = new Template("testName", sampleConfiguration);
        var s = spyOn(tpl, 'render');
        //expect(function(){
            tpl.dispatchRender({});
        //}).toThrow(Template.Error.NOT_IMPLEMENTED);
        expect(s).toHaveBeenCalledWith({});
    });
    it('dispatchRender / without children', function(){
        var sampleConfiguration = {
            template:"example/path",
            '#body':[]
        };
        var tpl = new Template("testName", sampleConfiguration);
        var s = spyOn(tpl, 'render');
        tpl.dispatchRender({});
        expect(s).toHaveBeenCalledWith({});
    });

    
    //TODO it('dispatchRender / with children', function(){});
    it('setRenderedData - parameters', function(){
        var tpl = new Template("test", {
            template: "blabla"
        });

        expect(function(){ tpl.setRenderedData(); }).toThrow(Template.Exception.WRONG_RENDER_DATA);
        expect(function(){ tpl.setRenderedData(null); }).toThrow(Template.Exception.WRONG_RENDER_DATA);
        expect(function(){ tpl.setRenderedData(undefined); }).toThrow(Template.Exception.WRONG_RENDER_DATA);
        expect(function(){ tpl.setRenderedData([]); }).toThrow(Template.Exception.WRONG_RENDER_DATA);
        expect(function(){ tpl.setRenderedData({}); }).toThrow(Template.Exception.WRONG_RENDER_DATA);
        expect(function(){ tpl.setRenderedData(new (function(){})); }).toThrow(Template.Exception.WRONG_RENDER_DATA);
        
        var jsonrpc = new JsonRpcMock("xx");
        expect(function(){ tpl.setRenderedData(jsonrpc); }).not.toThrow(Template.Exception.WRONG_RENDER_DATA);
    });


    
    it('setRenderedData', function(){
        runs(function(){
            var sampleConfiguration = {
                template:"example/path"
            };
            var self = this;
            
            this.test = 0;
            this.callbackF = function(e){
                self.test = 1;
            };
            
            this.rpc = new JsonRpcMock("xx");
            
            this.tpl = new Template("testName", sampleConfiguration);
            this.tpl.addEventListener(Template.RENDERED, this.callbackF, this);
            this.tpl.setRenderedData(this.rpc);
        });
        waits(1);
        runs(function(){
            expect(this.test).toEqual(1);
            expect(this.tpl.isRendered).toBeTruthy();
            expect(this.tpl.getContent().toJson().data).toEqual(this.rpc.toJson().data);
        });
    });



    it('setTemplateManager', function(){
        console.log("dispatchRender");
        var sampleConfiguration = {
            template:"example/path"
        };
        var tpl = new Template("testName", sampleConfiguration);
        expect(function(){
            tpl.setTemplateManager();
        }).toThrow(Template.Exception.TEMPLATEMANAGER_NULL);
        
        expect(function(){
            tpl.setTemplateManager(null);
        }).toThrow(Template.Exception.TEMPLATEMANAGER_NULL);
        
        expect(function(){
            tpl.setTemplateManager({});
        }).toThrow(Template.Exception.TEMPLATEMANAGER_NOT_CLASS_INSTANCE);
        
        var TemplateManager = require('../../lib/TemplateManager.js').TemplateManager; 
        
        var tm = new TemplateManager();
        expect(function(){
            tpl.setTemplateManager(tm);
        }).not.toThrow(Template.Error.TEMPLATEMANAGER_NOT_CLASS_INSTANCE);
    });
    
    it('addChild', function(){
        console.log("dispatchRender");
        var sampleConfiguration = {
            template:"example/path"
        };
        var parent = new Template("parent", sampleConfiguration);
        var child = new Template("child", sampleConfiguration);
        
        expect(child.hasParent).toBeFalsy();
        expect(child.hasEventListener(Template.RENDERED)).toBeFalsy();
        parent.addChild(child);
        expect(child.hasParent).toBeTruthy();
        expect(child.hasEventListener(Template.RENDERED)).toBeTruthy();
        
        expect(function(){
            parent.addChild(parent);
        }).toThrow(Template.Exception.RECURSIVE_TEMPLATES);

    });
});